package com.fjhx.victoriatourist.service.jd.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.fjhx.victoriatourist.entity.jd.dto.JdInfoDto;
import com.fjhx.victoriatourist.entity.jd.dto.JdInfoSelectDto;
import com.fjhx.victoriatourist.entity.jd.po.JdBack;
import com.fjhx.victoriatourist.entity.jd.po.JdBackDetails;
import com.fjhx.victoriatourist.entity.jd.po.JdOrder;
import com.fjhx.victoriatourist.entity.jd.po.JdOrderDetails;
import com.fjhx.victoriatourist.entity.jd.vo.JdInfoVo;
import com.fjhx.victoriatourist.service.jd.*;
import com.jd.open.api.sdk.DefaultJdClient;
import com.jd.open.api.sdk.JdClient;
import com.jd.open.api.sdk.domain.supplier.PoQueryForJosWebService.response.getdetailbyorderid.JOSDetailResultDto;
import com.jd.open.api.sdk.domain.supplier.PoQueryForJosWebService.response.getdetailbyorderid.PurchaseAllocationDetailDto;
import com.jd.open.api.sdk.domain.supplier.PoQueryForJosWebService.response.getpurchaseorderlist.JOSOrderResultDto;
import com.jd.open.api.sdk.domain.supplier.PoQueryForJosWebService.response.getpurchaseorderlist.PurchaseOrderDto;
import com.jd.open.api.sdk.domain.supplier.ReturnOrderForJosWebService.response.detail.DetailResultDto;
import com.jd.open.api.sdk.domain.supplier.ReturnOrderForJosWebService.response.get.RoDto;
import com.jd.open.api.sdk.domain.supplier.ReturnOrderForJosWebService.response.get.RoResultDto;
import com.jd.open.api.sdk.request.supplier.VcGetReturnOrderDetailRequest;
import com.jd.open.api.sdk.request.supplier.VcGetdetailbyorderidRequest;
import com.jd.open.api.sdk.request.supplier.VcGetpurchaseorderlistRequest;
import com.jd.open.api.sdk.request.supplier.VcReturnOrderListPageGetRequest;
import com.jd.open.api.sdk.response.supplier.VcGetReturnOrderDetailResponse;
import com.jd.open.api.sdk.response.supplier.VcGetdetailbyorderidResponse;
import com.jd.open.api.sdk.response.supplier.VcGetpurchaseorderlistResponse;
import com.jd.open.api.sdk.response.supplier.VcReturnOrderListPageGetResponse;
import com.ruoyi.system.utils.ConfigUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpRequestBase;
import org.apache.http.impl.client.HttpClients;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class JdApiServiceImpl implements JdApiService {

    private static final SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    private static final String appKey = "ECCD5CE82A501C7E0D8BB41C030C447D";
    private static final String appSecret = "5b55c3f3eb874b0d8746ff28e7f91f64";
    private static final String redirect_uri = "/open/jd/callback";
    private final String tenantId = "prod".equals(SpringUtil.getActiveProfile()) ? "wdly" : "wdlytest";
    private static final String ACCESS_TOKEN = "access_token";
    private static final String REFRESH_TOKEN = "refresh_token";

    @Autowired
    private JdOrderService jdOrderService;
    @Autowired
    private JdOrderDetailsService jdOrderDetailsService;
    @Autowired
    private JdBackService jdBackService;
    @Autowired
    private JdBackDetailsService jdBackDetailsService;
    @Resource
    private JdInfoService jdInfoService;

    /**
     * 获取AccessToken
     */
    @Override
    public void getAccessTokenByCode(String code) {
        //获取AccessToken并存入数据库
        String path = "https://open-oauth.jd.com/oauth2/access_token?app_key=%s&app_secret=%s&grant_type=authorization_code&code=%s";
        path = String.format(path, appKey, appSecret, code);
        try {
            JSONObject responseJson = getJSON(new HttpGet(path));
            dealJdResponse(responseJson);
        } catch (Exception e) {
            log.error("获取AccessToken出错：" + e.getMessage(), e);
        }
    }



    @Override
    public String build2LoginUrl() {
        String url = "https://open-oauth.jd.com/oauth2/to_login?app_key=%s&response_type=code&redirect_uri=%s&state=%s&scope=snsapi_base";
        //TODO redirect_uri需要加上项目的域名
        url = String.format(url, appKey, redirect_uri, DateUtil.format(new Date(), "yyyyMMddHHmmss"));
        return url;
    }

    /**
     * 刷新Token
     */
    private void refreshToken(JdInfoVo jdInfoVo) {
        if (ObjectUtil.isEmpty(jdInfoVo.getAccessToken())) {
            log.error("accessToken为空无法刷新Token请检查");
            return;
        }
        if (ObjectUtil.isEmpty(jdInfoVo.getRefreshToken())) {
            log.error("refreshToken为空无法刷新Token请检查");
            return;
        }
        String path = "https://open-oauth.jd.com/oauth2/refresh_token?app_key=%s&app_secret=%s&grant_type=refresh_token&refresh_token=%s";
        path = String.format(path, appKey, appSecret, jdInfoVo.getRefreshToken());
        try {
            JSONObject responseJson = getJSON(new HttpGet(path));
            dealJdResponse(responseJson);
        } catch (Exception e) {
            log.error("刷新AccessToken出错：" + e.getMessage(), e);
        }
    }

//    @DSTransactional
    //每小时执行一次
    @Scheduled(cron = "0 0 0/1  * * ? ")
    //每分钟执行一次
//    @Scheduled(cron = "0 0/1 *  * * ? ")
    void orderSync(){
        JdInfoVo jdInfoVo = jdInfoService.getOne();
        //如果过期时间减去当前时间小于90分钟，则要刷新token
        if(ObjectUtil.isNotNull(jdInfoVo.getExpireTime())){
            long between = DateUtil.between(DateUtil.date(), jdInfoVo.getExpireTime(), DateUnit.MINUTE, false);
            if(between <= 0){
                if(jdInfoVo.getOnline()>0){
                    jdInfoVo.setOnline(0);
                    jdInfoService.updateById(jdInfoVo);
                }
            }else if(90 >= between){
                refreshToken(jdInfoVo);
            }
        }

    }

    /**
     * 获取京东订单信息分页
     */
    private JOSOrderResultDto getJdOrderInfo(Integer pageIndex, Integer pageSize, Date startTime, Date endTime, JdClient client) throws Exception {
        VcGetpurchaseorderlistRequest request = new VcGetpurchaseorderlistRequest();
        request.setCreatedDateStart(startTime);
        request.setCreatedDateEnd(endTime);
        request.setPageIndex(pageIndex);
        request.setPageSize(pageSize);
        VcGetpurchaseorderlistResponse response = client.execute(request);
        return response.getJosOrderResultDto();
    }

    /**
     * 获取京东订单
     */
    private void getJdOrder(JdClient client) throws Exception {
        //获取开始同步的时间
        Date startTime = sdf.parse("2023-11-01 00:00:00");
        Date endTime = new Date();
        //根据下单时间获取数据库里最后一个订单
        JdOrder lastJdOrder = jdOrderService.getOne(q -> q.isNotNull(JdOrder::getCreatedDate).orderByDesc(JdOrder::getCreatedDate));
        if (ObjectUtil.isNotEmpty(lastJdOrder)) {
            //将最后一个下单时间减1个小时设置为开始时间
            startTime = calculateTime(lastJdOrder.getCreatedDate(),Calendar.HOUR_OF_DAY, -1);
        }
        //计算总页数
        Integer recordCount = getJdOrderInfo(1, 1, startTime, endTime, client).getRecordCount();
        int pageSize = 50;
        int pageCount = (recordCount + pageSize - 1) / pageSize;

        List<PurchaseOrderDto> purchaseOrderList = new ArrayList<>();
        //数据返回订单是倒序反过来按正顺保存
        for (int i = pageCount; i > 0; i--) {
            JOSOrderResultDto josOrderResultDto = getJdOrderInfo(i, pageSize, startTime, endTime, client);
            List<PurchaseOrderDto> purchaseOrderList1 = josOrderResultDto.getPurchaseOrderList();
            //京东回来的数据是按时间倒序的,使用reverse方法将list中的数据翻转为正序
            Collections.reverse(purchaseOrderList1);
            purchaseOrderList.addAll(purchaseOrderList1);
        }

        //处理数据（过滤已存在的数据）
        List<Long> jdOrderIds = purchaseOrderList.stream().map(PurchaseOrderDto::getOrderId).collect(Collectors.toList());
        //获取已同步的京东订单id
        List<Long> jdOrderIds1 = jdOrderService.listObject(JdOrder::getOrderId, q -> q.in(JdOrder::getOrderId, jdOrderIds));

        //过滤出未同步的数据
        List<PurchaseOrderDto> purchaseOrderListFilter = purchaseOrderList.stream().filter(item -> !jdOrderIds1.contains(item.getOrderId())).collect(Collectors.toList());

        //转为系统自己的实体类
        List<JdOrder> jdOrderList = BeanUtil.copyToList(purchaseOrderListFilter, JdOrder.class);
        //赋值租户id
        jdOrderList.forEach(item -> item.setTenantId(tenantId));
        //将数据存储到库
        jdOrderService.saveBatch(jdOrderList);

        //获取明细
        List<JdOrderDetails> jdOrderDetailsList = new ArrayList<>();
        for (JdOrder jdOrder : jdOrderList) {
            VcGetdetailbyorderidRequest request = new VcGetdetailbyorderidRequest();
            request.setOrderId(jdOrder.getOrderId());
            request.setIsPage(false);
            VcGetdetailbyorderidResponse response = client.execute(request);

            JOSDetailResultDto josDetailResultDto = response.getJosDetailResultDto();
            List<PurchaseAllocationDetailDto> purchaseAllocationDetailList = josDetailResultDto.getPurchaseAllocationDetailList();

            List<JdOrderDetails> jdOrderDetails = BeanUtil.copyToList(purchaseAllocationDetailList, JdOrderDetails.class);
            jdOrderDetails.forEach(item -> item.setJdOrderId(jdOrder.getId()));

            jdOrderDetailsList.addAll(jdOrderDetails);
        }
        //赋值租户id
        jdOrderList.forEach(item -> item.setTenantId(tenantId));
        jdOrderDetailsService.saveBatch(jdOrderDetailsList);
    }


    /**
     * 获取京东退货单信息分页
     */
    private static RoResultDto getJdBackOrderInfo(Integer pageIndex, Integer pageSize, Date startTime, Date endTime, JdClient client) throws Exception {
        VcReturnOrderListPageGetRequest request = new VcReturnOrderListPageGetRequest();
        request.setCreateDateBegin(startTime);
        request.setCreateDateEnd(endTime);
        request.setPageSize(pageIndex);
        request.setPageIndex(pageSize);
        VcReturnOrderListPageGetResponse response = client.execute(request);
        return response.getRoResultDto();
    }

    /**
     * 获取京东退货订单
     */
    private void getJdBackOrder(JdClient client) throws Exception {
        //获取开始同步的时间
        //获取开始同步的时间
        Date startTime = sdf.parse("2023-11-01 00:00:00");
        Date endTime = new Date();
        //根据下单时间获取数据库里最后一个订单
        JdBack lastJdBack = jdBackService.getOne(q -> q.isNotNull(JdBack::getCreateDate).orderByDesc(JdBack::getCreateDate));
        if (ObjectUtil.isNotEmpty(lastJdBack)) {
            //将最后一个下单时间减1个小时设置为开始时间
            startTime = calculateTime(lastJdBack.getCreateDate(),Calendar.HOUR_OF_DAY, -1);
        }

        //获取数据总页数
        RoResultDto jdBackOrderInfo = getJdBackOrderInfo(1, 1, startTime, endTime, client);
        Integer recordCount = jdBackOrderInfo.getRecordCount();
        int pageSize = 50;
        int pageCount = (recordCount + pageSize - 1) / pageSize;
        //获取最后一页数据
        RoResultDto jdBackOrderInfo1 = getJdBackOrderInfo(pageCount, pageSize, startTime, endTime, client);
        List<RoDto> roDtoList = jdBackOrderInfo1.getRoDtoList();
        //京东回来的数据是按时间倒序的,使用reverse方法将list中的数据翻转为正序
        Collections.reverse(roDtoList);

        //过滤出不在数据库里的新数据
        List<Long> returnIds = roDtoList.stream().map(RoDto::getReturnId).collect(Collectors.toList());
        List<Long> existReturnIds = jdBackService.listObject(JdBack::getReturnId, q -> q.in(JdBack::getReturnId, returnIds));
        roDtoList = roDtoList.stream().filter(item -> !existReturnIds.contains(item.getReturnId())).collect(Collectors.toList());
        //转成我们自己的实体类
        List<JdBack> jdBackList = BeanUtil.copyToList(roDtoList, JdBack.class);
        jdBackService.saveBatch(jdBackList);
        //获取明细
        List<JdBackDetails> JdBackDetailsList = new ArrayList<>();
        for (JdBack jdBack : jdBackList) {
            VcGetReturnOrderDetailRequest request = new VcGetReturnOrderDetailRequest();
            request.setReturnId(jdBack.getReturnId());
            VcGetReturnOrderDetailResponse response = client.execute(request);
            DetailResultDto detailResultDto = response.getDetailResultDto();

            List<JdBackDetails> jdBackDetails = BeanUtil.copyToList(detailResultDto.getDetailDtoList(), JdBackDetails.class);
            jdBackDetails.forEach(item -> item.setJdBackId(jdBack.getId()));
            JdBackDetailsList.addAll(jdBackDetails);
        }
        jdBackDetailsService.saveBatch(JdBackDetailsList);
    }


    /**
     * 请求API工具方法
     */
    private static JSONObject getJSON(HttpRequestBase httpRequestBase) throws IOException {
        HttpClient client = HttpClients.createDefault();
        httpRequestBase.setConfig(RequestConfig.custom().setConnectTimeout(3000).build());
        HttpResponse response = client.execute(httpRequestBase);
        int statusCode = response.getStatusLine().getStatusCode();
        HttpEntity entity = response.getEntity();
        String responseStr = IOUtils.toString(entity.getContent(), StandardCharsets.UTF_8);
        if (statusCode != 200) {
            log.error("请求失败 {} {} {}", statusCode, httpRequestBase.getURI(), responseStr);
        }
        return JSONObject.parseObject(responseStr);
    }

    /**
     * 计算时间公共方法
     */
    private Date calculateTime(Date date,Integer field, Integer amount){
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(field, amount);
        return calendar.getTime();
    }

    /**
     * 将jd返回的json对象转换成jdInfoVo
     * @author hj
     * @date 2023/12/10 22:03
     * @param responseJson
     * @param jdInfoVo
     */
    private void convertJdInfoVO(JSONObject responseJson, JdInfoVo jdInfoVo) {
        jdInfoVo.setAccessToken(responseJson.getString(ACCESS_TOKEN));
        jdInfoVo.setRefreshToken(responseJson.getString(REFRESH_TOKEN));
        int expires_in = responseJson.getIntValue("expires_in", 0);
        int offsetHour = 24;
        if(expires_in>0){
            offsetHour = expires_in/60/60;
        }
        jdInfoVo.setExpireTime(DateUtil.offsetHour(new Date(), offsetHour));
        jdInfoVo.setLastRefreshTime(DateUtil.date());
        jdInfoVo.setOnline(1);
    }

    /**
     * 处理jd返回对象
     * @author hj
     * @date 2023/12/10 22:07
     * @param responseJson 
     */
    private void dealJdResponse(JSONObject responseJson) {
        if (ObjectUtil.isNotEmpty(responseJson.getString(ACCESS_TOKEN)) && ObjectUtil.isNotEmpty(responseJson.getString(REFRESH_TOKEN))) {
            log.info("成功获取AccessToken并保存" + responseJson);
            JdInfoVo jdInfoVo = jdInfoService.getOne();
            convertJdInfoVO(responseJson, jdInfoVo);
            JdInfoDto jdInfoDto = BeanUtil.copyProperties(jdInfoVo, JdInfoDto.class);
            jdInfoService.add(jdInfoDto);
        } else {
            log.error("获取AccessToken失败：" + responseJson);
        }
    }

    public static void main(String[] args) throws Exception {
        JdClient client = new DefaultJdClient("http://api.jd.com/routerjson", "a1f79f4670fd43a28fa2de02abd86f03ytfm", appKey, appSecret);


    }

}
